﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;

namespace Spread_CutomCellType_demo
{
    /// <summary>
    /// 对角线CellType
    /// 继承自TextCellType
    /// </summary>
    public class DiagonalCellType : FarPoint.Win.Spread.CellType.TextCellType
    {
        #region CTOR
        SubDiagonalCellTypeBase m_cellType = null;
        public DiagonalCellType(DiagonalSubCellType subType)
        {
            m_cellType = SubDiagonalCellTypeBase.CreateSubType(subType);
        }
        #endregion

        /// <summary>
        /// 添加Text，双区域需要添加2次Text，三区域需要添加3次Text，四区域需要添加4次Text
        /// </summary>
        /// <param name="text">绘制文本</param>
        /// <param name="sstringAlignment">Alignment</param>
        /// <param name="ssStringFormatFlags">FormatFlags</param>
        public void SetText(string text, StringAlignment sstringAlignment = StringAlignment.Near, StringFormatFlags ssStringFormatFlags = StringFormatFlags.NoClip)
        {
            m_cellType.SetText(text, sstringAlignment, ssStringFormatFlags);
        }

        /// <summary>
        /// 自绘事件
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        /// <param name="appearance"></param>
        /// <param name="value"></param>
        /// <param name="isSelected"></param>
        /// <param name="isLocked"></param>
        /// <param name="zoomFactor"></param>
        public override void PaintCell(Graphics g, Rectangle r, Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor)
        {
            m_cellType.PaintCell(g, r, appearance, value, isSelected, isLocked, zoomFactor);
        }

        /// <summary>
        /// 开始启动编辑器
        /// </summary>
        /// <param name="e"></param>
        /// <param name="selectAll"></param>
        /// <param name="autoClipboard"></param>
        public override void StartEditing(EventArgs e, bool selectAll, bool autoClipboard)
        {
            base.StartEditing(e, selectAll, autoClipboard);
            m_editor.Multiline = true;
            m_editor.Clear();
            string[] strs = m_cellType.GetTextsCore();
            for (int i = 0; i < strs.Length; i++)
            {
                m_editor.AppendText(strs[i]);
                if (i != strs.Length -1)
                {
                    m_editor.AppendText(Environment.NewLine);
                }
            }
            
            m_editor.Text.Trim();
        }

        /// <summary>
        /// 完成编辑
        /// </summary>
        public override void CancelEditing()
        {
            base.CancelEditing();

            string[] strs = m_editor.Text.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            m_cellType.SetTextsCore(strs);
        }

        /// <summary>
        /// 获得编辑器
        /// </summary>
        /// <param name="parent"></param>
        /// <param name="appearance"></param>
        /// <param name="zoomFactor"></param>
        /// <returns></returns>
        public override System.Windows.Forms.Control GetEditorControl(System.Windows.Forms.Control parent, Appearance appearance, float zoomFactor)
        {
            return m_editor;
        }

        private System.Windows.Forms.TextBox m_editor = new System.Windows.Forms.TextBox();

        /// <summary>
        /// Line的颜色，默认为ForeColor
        /// </summary>
        public Color LineColor
        {
            get { return m_cellType.LineColor; }
            set { m_cellType.LineColor = value; }
        }

        /// <summary>
        /// 对角线Line宽度，默认为1f
        /// </summary>
        public float LineWidth
        {
            get { return m_cellType.LineWidth; }
            set { m_cellType.LineWidth = value; }
        }

        
    }
}
