﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using FarPoint.Win.Spread;

namespace Spread_CutomCellType_demo
{
    /// <summary>
    /// 对角线CellType实现类
    /// </summary>
    public abstract class SubDiagonalCellTypeBase
    {
        /// <summary>
        /// CTOR
        /// </summary>
        public SubDiagonalCellTypeBase()
        {
            LineColor = Color.Empty;
            LineWidth = 1f;
        }

        /// <summary>
        /// 工厂类创建子对角线类型
        /// </summary>
        /// <param name="subType"></param>
        /// <returns></returns>
        public static SubDiagonalCellTypeBase CreateSubType(DiagonalSubCellType subType)
        {
            switch (subType)
            {
                case DiagonalSubCellType.e2Region:
                    return new SubDiagonalCellType_2Region();
                case DiagonalSubCellType.e3Region:
                    return new SubDiagonalCellType_3Region();
                case DiagonalSubCellType.e4Region:
                    return new SubDiagonalCellType_4Region();
                default:
                    break;
            }
            return null;
        }

        /// <summary>
        /// 模板方法实现PainCell
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        /// <param name="appearance"></param>
        /// <param name="value"></param>
        /// <param name="isSelected"></param>
        /// <param name="isLocked"></param>
        /// <param name="zoomFactor"></param>
        public void PaintCell(Graphics g, Rectangle r, Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor)
        {
            ValidateData(m_TextList.Count);

            using (Brush SolidBrush = new SolidBrush(appearance.BackColor))
            {
                g.FillRectangle(SolidBrush, r);
            }

            if (LineColor == Color.Empty)
            {
                LineColor = appearance.ForeColor;
            }
            using (Pen LinePen = new Pen(LineColor, LineWidth))
            {
                InnerPaintCell(g, r, appearance, value, isSelected, isLocked, zoomFactor, LinePen);
            }
        }

        /// <summary>
        /// 子类实现绘制的核心函数
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        /// <param name="appearance"></param>
        /// <param name="value"></param>
        /// <param name="isSelected"></param>
        /// <param name="isLocked"></param>
        /// <param name="zoomFactor"></param>
        /// <param name="LinePen"></param>
        public abstract void InnerPaintCell(Graphics g, Rectangle r, Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor, Pen LinePen);

        /// <summary>
        /// 数据校验
        /// </summary>
        /// <param name="count"></param>
        protected abstract void ValidateData(int count);

        /// <summary>
        /// 当前对角线类型：枚举
        /// </summary>
        public abstract DiagonalSubCellType DiagonalSubCellType { get; }

        /// <summary>
        /// Text包装器
        /// </summary>
        /// <param name="text"></param>
        /// <param name="sstringAlignment"></param>
        /// <param name="ssStringFormatFlags"></param>
        public virtual void SetText(string text, StringAlignment sstringAlignment = StringAlignment.Near, StringFormatFlags ssStringFormatFlags = StringFormatFlags.NoClip)
        {
            TextWrap txtWrap = new TextWrap() { Text=text,StringAlignment=sstringAlignment, StringFormatFlags=ssStringFormatFlags};
            m_TextList.Add(txtWrap);
        }

        /// <summary>
        /// Line颜色
        /// </summary>
        public Color LineColor { get; set; }

        /// <summary>
        /// Line宽度
        /// </summary>
        public float LineWidth { get; set; }

        protected List<TextWrap> m_TextList = new List<TextWrap>();

        /// <summary>
        /// 获得当前Text数组
        /// </summary>
        /// <returns></returns>
        public string[] GetTextsCore()
        {
            List<string> list = new List<string>();
            foreach (var item in m_TextList)
            {
                list.Add(item.Text);
            }
            return list.ToArray();
        }

        /// <summary>
        /// 设置当前Text数组，个数必须和对角线类型一致，否则本次设置无效
        /// </summary>
        /// <param name="texts"></param>
        public void SetTextsCore(string[] texts)
        {
            try
            {
                ValidateData(texts.Length);
            }
            catch (Exception)
            {
                return;
            }

            for (int i = 0; i < texts.Length; i++)
            {
                m_TextList[i].Text = texts[i];   
            }            
        }
    }


}
